import requests
import xml.etree.ElementTree as ET

# Functie om handelingen te ontsluiten
# Deze functie voert een request uit naar de SRU API van de overheid en geeft de resultaten weer in de notebook
def ontsluit_handelingen(query_part2, startRecord=1, maximumRecords=10):
    
    # Connectiestring met basis parameters, waarbij al gefilterd wordt op 'officielepublicaties'
    url = "https://repository.overheid.nl/sru"
    params = {
    "operation": "searchRetrieve",
    "version": "2.0",
    "query": "c.product-area==officielepublicaties",  
    "startRecord": startRecord,
    "maximumRecords": maximumRecords,
    "recordSchema": "gzd"
    }
    
    # Als er een tweede deel voor de query is, voeg deze toe aan de originele query en vervang de query in de parameters
    if query_part2:
        params["query"] += query_part2
    
    # Voer de request uit
    response = requests.get(url, params=params)

    # Controleer of de request gelukt is
    if response.status_code == 200:
        # Zet de response om naar een XML object (bytes)
        xml_data = response.content

        # Parse de XML vanuit bytes naar een ElementTree object
        root = ET.fromstring(xml_data)
        
        # Return het ElementTree object
        return root
    else:
        print(f"Request failed with status code: {response.status_code}")


from .structure import OfficielePublicatie
from .webservice import ontsluit_handelingen

def retrieve_publications(query_list = [], max_records = 10, start_record = 1):
    query_concatter = ' AND '
    query_part2 = ''
    for value in query_list:
        query_part2 += f"{query_concatter}{value}"
    
    root = ontsluit_handelingen(query_part2, start_record, max_records)
    
    namespaces = {
    'sru': "http://docs.oasis-open.org/ns/search-ws/sruResponse",
    'gzd': "http://standaarden.overheid.nl/sru"
    }
    
    ops = []
    # Loop door iedere record in de records container
    for record in root.findall('.//sru:records/sru:record', namespaces):
        # Genest in <sru:recordData>
        record_data = record.find('.//sru:recordData', namespaces)
        if record_data is not None:
            ob = OfficielePublicatie.from_xml_element(record_data, namespaces)
            ops.append(ob)
            
    return ops


import xml.etree.ElementTree as ET

class OfficielePublicatie:
    def __init__(self):
        self.identifier = []
        self.title = []
        self.type = []
        self.language = []
        self.authority = []
        self.creator = []
        self.modified = []
        self.temporal = []
        self.spatial = []
        self.alternative = []
        self.date = []
        self.hasVersion = []
        self.source = []
        self.requires = []
        self.isPartOf = []
        self.isrequiredby = []
        self.isreplacedby = []
        self.hasPart = []
        self.subject = []
        self.available = []
        self.abstract = []
        self.publisher = []
        self.issued = []
        self.replaces = []
        self.aanhangsel = []
        self.aanhangselnummer = []
        self.adviesRvs = []
        self.bedrijfsnaam = []
        self.behandeldDossier = []
        self.betreft = []
        self.betreftopschrift = []
        self.betreftRegeling = []
        self.bijlage = []
        self.datumBrief = []
        self.datumIndiening = []
        self.datumOndertekening = []
        self.datumOntvangst = []
        self.datumTotstandkoming = []
        self.datumVergadering = []
        self.deurwaardersdossier = []
        self.documentstatus = []
        self.documenttitel = []
        self.dossiernummer = []
        self.dossiertitel = []
        self.effectgebied = []
        self.einddatum = []
        self.datumEindeReactietermijn = []
        self.eindpagina = []
        self.externeBijlage = []
        self.gebiedsmarkering = []
        self.gemeentenaam = []
        self.geometrie = []
        self.geometrie = []
        self.geometrielabel = []
        self.hectometernummer = []
        self.heeftMededeling = []
        self.hoofddocument = []
        self.huisnummer = []
        self.huisletter = []
        self.huisnummertoevoeging = []
        self.indiener = []
        self.handelingenitemnummer = []
        self.jaargang = []
        self.kadastraleSectie = []
        self.ketenid = []
        self.ligtInGemeente = []
        self.ligtInProvincie = []
        self.materieelUitgewerkt = []
        self.mededelingOver = []
        self.ondernummer = []
        self.ontvanger = []
        self.organisatietype = []
        self.perceelnummer = []
        self.persoonsgegevens = []
        self.plaatsTotstandkoming = []
        self.postcode = []
        self.postcodeHuisnummer = []
        self.provincie = []
        self.provincienaam = []
        self.publicatienummer = []
        self.publicatienaam = []
        self.besluitReferendabiliteit = []
        self.referentienummer = []
        self.rijkswetnummer = []
        self.bekendmakingBetreffendePlan = []
        self.startdatum = []
        self.startpagina = []
        self.straatnaam = []
        self.subrubriek = []
        self.sysyear = []
        self.sysnumber = []
        self.sysseqnumber = []
        self.terinzageleggingBG = []
        self.terinzageleggingOP = []
        self.typeVerkeersbesluit = []
        self.verdragnummer = []
        self.vereisteVanBesluit = []
        self.vergaderjaar = []
        self.verkeersbordcode = []
        self.versieinformatie = []
        self.versienummer = []
        self.vraagnummer = []
        self.waterschapsnaam = []
        self.wegcategorie = []
        self.weggebruiker = []
        self.wegnummer = []
        self.woonplaats = []
        self.zittingsdatum = []
        self.product_area = []
        self.content_area = []
        self.datumTijdstipWijzigingWork = []
        self.datumTijdstipWijzigingExpression = []
        self.url = []
        self.prefferedUrl = []
        self.itemUrl = []
        self.timestamp = []
        
    @classmethod
    def from_xml_element(cls, element: ET.Element, namespaces: dict) -> 'OfficielePublicatie':
        op = cls()
        for child in element.findall('.//*', namespaces):
            tag = cls.parse_namespaced_tag(child.tag)

            if tag not in op.__dict__:
                continue

            if child.attrib:
                key, value = list(child.attrib.items())[0]
                value_dict = {value: child.text.strip() if child.text else ""}
                # value_dict['value'] = child.text.strip() if child.text else ""
                op.__dict__[tag].append(value_dict)
            else:
                op.__dict__[tag].append(child.text.strip() if child.text else "")

        return op

    @staticmethod
    def parse_namespaced_tag(tag: str) -> str:
        return tag.split('}', 1)[-1] if '}' in tag else tag

    def __iter__(self):
        for key in self.__dict__:
            yield {key: getattr(self, key)}

    def __repr__(self):
        return f"<OfficielePublicatie identifier={self.identifier}, title={self.title}>"


from setuptools import find_packages, setup

with open('README.md') as f:
    long_description = f.read()

setup(
    name='PyPublicaties',
    packages=find_packages(include=['PyPublicaties']),
    version='0.2.3',
    description='A Python library which can be used to interact with the Dutch government\'s publication repository.',
    author='Pascal Venema',
    install_requires=['requests'],
    long_description=long_description,
    long_description_content_type='text/markdown',
)